require 'spec_helper'

module Nestene

  describe FileNameConverter do
    describe '#key_to_file_name' do
      it "should convert non-charater in the file name to hex-escaped version" do
        expect(subject.key_to_file_name('test_-+/')).to eq('test_5f_2d_2b_2f')
      end

    end

    describe '#file_name_to_key' do
      it "should convert hex-escaped version of the file name to the original" do
        expect(subject.file_name_to_key('test_5f_2d_2b_2f')).to eq('test_-+/')
      end
    end
  end

  describe DiskStorage do

    let(:tmpdir) {Dir.mktmpdir}

    after{FileUtils.remove_entry_secure tmpdir}

    subject {DiskStorage.new(tmpdir)}

    describe :list do

      context "when there is nothing in the storage" do
        it "should return empty list" do
          expect(subject.list).to eq([])
        end
      end

      context "when there is at least one object in the storage" do
        before {subject.store("test",{})}

        it "should return list of existing objects" do
          expect(subject.list).to eq(['test'])
        end
      end
    end

    describe :load do

      context "when the object does not exist" do
        it "should return nil" do
          expect(subject.load('whatever')).to be_nil
        end
      end

      context "when the object exists" do
        before {subject.store('test',{})}
        it "should return object's structure" do
          expect(subject.load('test')).to eq({})
        end
      end

    end

    describe :store do
      it "should store structure" do
        subject.store('test',{})
        expect(subject.load('test')).to eq({})
      end

      context "when structure is nil" do
        it 'should delete the entry' do
          subject.store('test',{})
          subject.store('test',nil)
          expect(subject.load('test')).to eq(nil)
          expect(subject.list).to_not include('test')
        end
      end
    end

    describe :delete do
      it "deletes existing strucuture" do
        subject.store('test',{})
        subject.delete('test')
        expect(subject.load('test')).to eq(nil)
        expect(subject.list).to_not include('test')
      end
    end

  end

  describe MemoryStorage do

    subject {MemoryStorage.new}

    describe :list do

      context "when there is nothing in the storage" do
        it "should return empty list" do
          expect(subject.list).to eq([])
        end
      end

      context "when there is at least one object in the storage" do
        before {subject.store("test",{})}
        it "should return list of existing objects" do
          expect(subject.list).to eq(['test'])
        end
      end
    end

    describe :load do
      context "when the object does not exist" do
        it "should return nil" do
          expect(subject.load('whatever')).to be_nil
        end
      end

      context "when the object exists" do
        before {subject.store('test',{})}
        it "should return object's structure" do
          expect(subject.load('test')).to eq({})
        end
      end
    end


    describe :store do
      it "should store structure" do
        subject.store('test',{})
        expect(subject.load('test')).to eq({})
      end

      context "when structure is nil" do
        it 'should delete the entry' do
          subject.store('test',{})
          subject.store('test',nil)
          expect(subject.load('test')).to eq(nil)
          expect(subject.list).to_not include('test')
        end
      end

    end

    describe :delete do
      it "deletes existing strucuture" do
        subject.store('test',{})
        subject.delete('test')
        expect(subject.load('test')).to eq(nil)
        expect(subject.list).to_not include('test')
      end
    end

  end
end